@@ -1,5 +1,5 @@
 /*
- *  $Id: fallback.c,v 5.53 2003-03-17 08:27:34-08 bryan Exp $
+ *  $Id: fallback.c,v 5.58 2003-08-15 08:18:26-07 bryan Exp $
  *
  *  Copyright conserver.com, 2000
  *
@@ -14,78 +14,31 @@
  * Mike Rowan (mtr@mace.cc.purdue.edu)
  */
 
-#include <config.h>
-
-#include <sys/param.h>
-#include <sys/stat.h>
-#include <sys/file.h>
-#include <fcntl.h>
-#include <sys/socket.h>
-#include <sys/errno.h>
-#include <netinet/in.h>
-#include <syslog.h>
-#include <signal.h>
-#include <netdb.h>
-#include <stdio.h>
-#include <ctype.h>
-
 #include <compat.h>
+
 #include <util.h>
 
-#if defined(_AIX)
 /*
- * get a pty for the user (emulate the neato sequent call)		(mm)
+ * get a pty for the user
+ *
+ * this has been revamped rather heavily for 8.0.0.  i've taken ideas
+ * from the xemacs and openssh distributions to get code that *should*
+ * work on systems i have no access to.  thanks to those reference
+ * packages, i think things are ok...hopefully it's true!
  */
 static int
 #if PROTOTYPES
-GetPseudoTTY(STRING * slave, STRING * master)
+GetPseudoTTY(STRING *slave, int *slaveFD)
 #else
-GetPseudoTTY(slave, master)
+GetPseudoTTY(slave, slaveFD)
     STRING *slave;
-    STRING *master;
+    int *slaveFD;
 #endif
 {
-    int fd;
-    char *pcName;
-
-    if (0 > (fd = open("/dev/ptc", O_RDWR | O_NDELAY, 0))) {
-	return -1;
-    }
-    if ((char *)0 == (pcName = ttyname(fd))) {
-	return -1;
-    }
-    BuildString((char *)0, slave);
-    BuildString(pcName, slave);
-
-    BuildString((char *)0, master);
-    BuildString(pcName, master);
-    master->string[7] = 'c';
-
-    return fd;
-}
-
-#else
-
-#if defined(HAVE_PTSNAME) && defined(HAVE_GRANTPT) && defined(HAVE_UNLOCKPT)
-#if defined(linux)
-extern char *ptsname();
-extern int grantpt();
-extern int unlockpt();
-#endif
-
-/* get a pty for the user -- emulate the neato sequent call under	(gregf)
- * DYNIX/ptx v4.0
- */
-static int
-#if PROTOTYPES
-GetPseudoTTY(STRING * slave, STRING * master)
-#else
-GetPseudoTTY(slave, master)
-    STRING *slave;
-    STRING *master;
-#endif
-{
-    int fd;
+#if HAVE_OPENPTY
+    int fd = -1;
+    int sfd = -1;
+    int opty = 0;
     char *pcName;
 #if HAVE_SIGACTION
     sigset_t oldmask, newmask;
@@ -93,9 +46,6 @@ GetPseudoTTY(slave, master)
     extern RETSIGTYPE FlagReapVirt PARAMS((int));
 #endif
 
-    if (0 > (fd = open("/dev/ptmx", O_RDWR, 0))) {
-	return -1;
-    }
 #if HAVE_SIGACTION
     sigemptyset(&newmask);
     sigaddset(&newmask, SIGCHLD);
@@ -106,7 +56,7 @@ GetPseudoTTY(slave, master)
     SimpleSignal(SIGCHLD, SIG_DFL);
 #endif
 
-    grantpt(fd);		/* change permission of slave */
+    opty = openpty(&fd, &sfd, NULL, NULL, NULL);
 
 #if HAVE_SIGACTION
     if (sigprocmask(SIG_SETMASK, &oldmask, NULL) < 0)
@@ -116,52 +66,121 @@ GetPseudoTTY(slave, master)
     SimpleSignal(SIGCHLD, FlagReapVirt);
 #endif
 
-    unlockpt(fd);		/* unlock slave */
-    BuildString((char *)0, master);
-    if ((char *)0 == (pcName = ttyname(fd))) {
-	BuildString("/dev/ptmx", master);
-    } else {
-	BuildString(pcName, master);
+    if (opty != 0) {
+	if (fd >= 0)
+	    close(fd);
+	if (sfd >= 0)
+	    close(sfd);
+	return -1;
     }
+    if ((char *)0 == (pcName = ttyname(sfd))) {
+	close(fd);
+	close(sfd);
+	return -1;
+    }
+    BuildString((char *)0, slave);
+    BuildString(pcName, slave);
 
-    if ((char *)0 == (pcName = ptsname(fd))) {
+    *slaveFD = sfd;
+    return fd;
+#else
+#if (HAVE_PTSNAME && HAVE_GRANTPT && HAVE_UNLOCKPT) || defined(_AIX)
+    int fd = -1;
+    int sfd = -1;
+    char *pcName;
+#if HAVE_SIGACTION
+    sigset_t oldmask, newmask;
+#else
+    extern RETSIGTYPE FlagReapVirt PARAMS((int));
+#endif
+    int c;
+    /* clone list and idea stolen from xemacs distribution */
+    static char *clones[] = {
+	"/dev/ptmx",		/* Various systems */
+	"/dev/ptm/clone",	/* HPUX */
+	"/dev/ptc",		/* AIX */
+	"/dev/ptmx_bsd",	/* Tru64 */
+	(char *)0
+    };
+
+    /* try to find the pty allocator */
+    for (c = 0; clones[c] != (char *)0; c++) {
+	if ((fd = open(clones[c], O_RDWR, 0)) >= 0)
+	    break;
+    }
+    if (fd < 0)
+	return -1;
+
+#if HAVE_SIGACTION
+    sigemptyset(&newmask);
+    sigaddset(&newmask, SIGCHLD);
+    if (sigprocmask(SIG_BLOCK, &newmask, &oldmask) < 0)
+	Error("GetPseudoTTY(): sigprocmask(SIG_BLOCK): %s",
+	      strerror(errno));
+#else
+    SimpleSignal(SIGCHLD, SIG_DFL);
+#endif
+
+#if HAVE_GRANTPT
+    grantpt(fd);		/* change permission of slave */
+#endif
+
+#if HAVE_SIGACTION
+    if (sigprocmask(SIG_SETMASK, &oldmask, NULL) < 0)
+	Error("GetPseudoTTY(): sigprocmask(SIG_SETMASK): %s",
+	      strerror(errno));
+#else
+    SimpleSignal(SIGCHLD, FlagReapVirt);
+#endif
+
+#if HAVE_UNLOCKPT
+    unlockpt(fd);		/* unlock slave */
+#endif
+
+#if defined(_AIX)
+    if ((pcName = ttyname(fd)) == (char *)0) {
+	close(fd);
+	return -1;
+    }
+#else
+# if HAVE_PTSNAME
+    if ((pcName = ptsname(fd)) == (char *)0) {
+	close(fd);
+	return -1;
+    }
+# else
+    close(fd);
+    return -1;
+# endif
+#endif
+
+    /* go ahead and open the slave */
+    if ((sfd = open(pcName, O_RDWR, 0)) < 0) {
+	Error("GetPseudoTTY(): open(%s): %s", pcName, strerror(errno));
+	close(fd);
 	return -1;
     }
 
     BuildString((char *)0, slave);
     BuildString(pcName, slave);
 
+    *slaveFD = sfd;
     return fd;
-}
 #else
-/*
- * Below is the string for finding /dev/ptyXX.  For each architecture we
- * leave some pty's world writable because we don't have source for
- * everything that uses pty's.  For the most part, we'll be trying to
- * make /dev/ptyq* the "free" pty's.
- */
-
-/* all the world's a vax ;-) */
-static char charone[] = "prstuvwxyzPQRSTUVWq";
-static char chartwo[] =
-    "0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ";
-
-/*
- * get a pty for the user (emulate the neato sequent call)		(ksb)
- */
-static int
-#if PROTOTYPES
-GetPseudoTTY(STRING * slave, STRING * master)
-#else
-GetPseudoTTY(slave, master)
-    STRING *slave;
-    STRING *master;
-#endif
-{
+    /*
+     * Below is the string for finding /dev/ptyXX.  For each architecture we
+     * leave some pty's world writable because we don't have source for
+     * everything that uses pty's.  For the most part, we'll be trying to
+     * make /dev/ptyq* the "free" pty's.
+     */
+    /* all the world's a vax ;-) */
+    static char charone[] = "prstuvwxyzPQRSTUVWq";
+    static char chartwo[] =
+	"0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ";
     static char acMaster[] = "/dev/ptyXX";
     static char acSlave[] = "/dev/ttyXX";
     static char *pcOne = charone, *pcTwo = chartwo;
-    int fd, iLoop, iIndex = sizeof("/dev/pty") - 1;
+    int fd, sfd, iLoop, iIndex = sizeof("/dev/pty") - 1;
     char *pcOld1;
     struct stat statBuf;
 
@@ -203,41 +222,47 @@ GetPseudoTTY(slave, master)
 	break;
     }
 
-    BuildString((char *)0, master);
-    BuildString(acMaster, master);
+    /* go ahead and open the slave */
+    if ((sfd = open(acSlave, O_RDWR, 0)) < 0) {
+	Error("GetPseudoTTY(): open(%s): %s", acSlave, strerror(errno));
+	close(fd);
+	return -1;
+    }
+
     BuildString((char *)0, slave);
     BuildString(acSlave, slave);
+
+    *slaveFD = sfd;
     return fd;
+#endif /* (HAVE_PTSNAME && HAVE_GRANTPT && HAVE_UNLOCKPT) || defined(_AIX) */
+#endif /* HAVE_OPENPTY */
 }
-#endif
-#endif /* _AIX */
 
 /*
- * get a Joe pty bacause the daemon is not with us, sadly.		(ksb)
+ * get a pty using the GetPseudoTTY code above
  */
 int
 #if PROTOTYPES
-FallBack(STRING * pcSlave, STRING * pcMaster)
+FallBack(char **slave, int *sfd)
 #else
-FallBack(pcSlave, pcMaster)
-    STRING *pcSlave, *pcMaster;
+FallBack(slave, sfd)
+    char **slave;
+    int *sfd;
 #endif
 {
     int fd;
-    static STRING *pcTSlave = (STRING *) 0;
-    static STRING *pcTMaster = (STRING *) 0;
+    static STRING *pcTSlave = (STRING *)0;
 
-    if (pcTSlave == (STRING *) 0)
+    if (pcTSlave == (STRING *)0)
 	pcTSlave = AllocString();
-    if (pcTMaster == (STRING *) 0)
-	pcTMaster = AllocString();
 
-    if (-1 == (fd = GetPseudoTTY(pcTSlave, pcTMaster))) {
+    if ((fd = GetPseudoTTY(pcTSlave, sfd)) == -1) {
 	return -1;
     }
-    BuildString((char *)0, pcSlave);
-    BuildString(pcTSlave->string, pcSlave);
-    BuildString((char *)0, pcMaster);
-    BuildString(pcTMaster->string, pcMaster);
+    if ((*slave) != (char *)0)
+	free(*slave);
+    if (((*slave) = strdup(pcTSlave->string))
+	== (char *)0)
+	OutOfMem();
     return fd;
 }
